//
//  NSString+OJAKit.m
//  OJAKitDemo
//
//  Created by luxiaoming on 16/7/4.
//  Copyright © 2016年 luxiaoming. All rights reserved.
//

#import "NSString+OJAKit.h"

@implementation NSString (OJAKit)


- (CGFloat)heightForWidth:(CGFloat)width withAttributes:(nullable NSDictionary<NSString *, id> *)attributes {
    CGSize maxSize = CGSizeMake(width, CGFLOAT_MAX);
    return [self sizeForMaxSize:maxSize withAttributes:attributes].height;
}

/**
 *  用系统的boundsWithRect的方法计算出来的string高度，并用ceil函数处理返回结果
 *  用style设置行高的方式，可以比较完美的解决兼容emoji表情的问题
 *  注意！！！这个方法强制将NSLineBreakByTruncatingTail转换成了NSLineBreakByWordWrapping来计算（如果存在的话）
 */
- (CGSize)sizeForMaxSize:(CGSize)maxSize withAttributes:(nullable NSDictionary<NSString *, id> *)attributes {
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithDictionary:attributes];
    NSMutableParagraphStyle *style = [[dict objectForKey:NSParagraphStyleAttributeName] mutableCopy];
    if (style && style.lineBreakMode == NSLineBreakByTruncatingTail) {
        style.lineBreakMode = NSLineBreakByWordWrapping;
        [dict setObject:style forKey:NSParagraphStyleAttributeName];
    }
    
    CGRect rect = [self boundingRectWithSize:maxSize options:NSStringDrawingUsesLineFragmentOrigin | NSStringDrawingUsesFontLeading attributes:dict context:nil];
    return CGSizeMake(ceil(rect.size.width), ceil(rect.size.height));
}

/**
 *  判断string是否合法，如何不为nil且length>0返回yes，其他返回NO
 */
- (BOOL)isValidString {
    if (![self isKindOfClass:[NSString class]]) {
        return NO;
    }
    if (self && self.length > 0) {
        return YES;
    } else {
        return NO;
    }
}

@end
